#!/bin/sh
#
# exportusers
#
# USAGE: see usage_()
#


BaseDir=$( dirname "$0" )
ColorShi="color.shi"
[ -n "${BaseDir}" ] && ColorShi="${BaseDir}/${ColorShi}" 

. $HOME/etc/setenv.sh

SqlSh="$TOOLS_HOME/sql.sh"
MysqlDump="$MYSQL_HOME/bin/mysqldump"

WorkDir="$TMP/eu.$$"
WorkDumpDir="${WorkDir}/DBDUMPS"
WorkUserRepo="${WorkDir}/USERREPO"
InfoFile="${WorkDir}/info.txt"


[ -f "${ColorShi}" ] && source "${ColorShi}"

#
# Prints usage information and exit
#
# [$1]: error message
#
function usage_() {
	
	if [ -n "$1" ] ; then
		error_ "$1"
		echo
	fi

	# regular color
	local reg="${ColorNormal}"
	# option color
	local opt="${ColorYellow}"
	# option argument color
	local arg="${ColorLightMagenta}"
	# reset color
	local end="${FontReset}"
	# file name color
	local fn="${ColorLightBlue}"
	# literal values
	local lit="${ColorCyan}"

	echo -e -n "${reg}exportusers.sh"
	echo -e " [${opt}-h${reg}] [${opt}-v${reg}] [${opt}-c ${arg}color${reg}] [${opt}-f ${arg}file${reg}]"
	echo -e -n "     [${opt}-s ${arg}system user mode${reg}]"
	echo -e " [${opt}-x ${arg}support user mode${reg}]"
	echo -e -n "     [${opt}-l ${arg}location mode${reg}]"
	echo -e -n " [${opt}-r ${arg}practice mode${reg}]"
	echo -e " [${opt}-m ${arg}managed groups mode${reg}]"
	echo -e -n "     [${opt}-p ${arg}plugins mode${reg}]"
	echo -e " [${opt}-u ${arg}user group mode${reg}]"
	echo -e -n "     [${opt}-a${reg} ] [${opt}-g${reg} ${arg}group${reg}]..."
	echo -e " [${arg}id${reg}...]${end}"

	echo

	echo -e "  ${reg}Generic options${end}"
	echo -e "      ${opt}-h${reg}         : Prints this help${end}"
	echo -e "      ${opt}-v${reg}         : Prints verbose messages${end}"
	echo -e "      ${opt}-c${arg} color${reg}   : Color mode (${lit}auto${reg} (default), ${lit}never${reg}, ${lit}always${reg})${end}"
	echo -e "      ${opt}-f${arg} file${reg}    : The export file name (default: ${fn}eu.tar.gz${reg})${end}"
	echo
	echo -e "  ${reg}Users/Groups to export${end}"
	echo -e "      ${arg}id${reg}...      : List of user and group IDs to export. Group IDs starts with '${lit}#${reg}'."
	echo -e "                   Group members for groups listed here will not be exported."
	echo -e "      ${opt}-a${reg}         : Export all users and groups. No ${arg}id${reg} list shall be provided.${end}"
	echo -e "      ${opt}-g${arg} group${reg}   : Export all users from the provided group including group data.${end}"
	echo -e "                   Several ${opt}-g ${arg}group${reg} can be specified.${end}"
	echo -e "      ${opt}-u${reg}         : Controls exporting group data for individual users in ${arg}id${reg} list.${end}"
	echo -e "           ${reg}Possible modes for export User Group settings:${end}"
	echo -e "               ${lit}yes${reg}   : Also export user group data.${end}"
	echo -e "               ${lit}no${reg}    : Do not export group data (default).${end}"
	echo
	echo -e "  ${reg}System/Support user settings${end}"
	echo -e "      ${opt}-s${arg} mode${reg}    : Controls exporting ${lit}@system${reg} user settings.${end}"
	echo -e "      ${opt}-x${arg} mode${reg}    : Controls exporting ${lit}@support${reg} user settings.${end}"
	echo -e "           ${reg}Possible modes for System/Support user settings:${end}"
	echo -e "               ${lit}auto${reg}  : Export system/support user settings only when exporting all users (${opt}-a${reg}) (default).${end}"
	echo -e "               ${lit}no${reg}    : Do not export system/support user settings.${end}"
	echo -e "               ${lit}yes${reg}   : Export system/support user settings.${end}"
	echo
	echo -e "  ${reg}Location/Practice/Managed groups settings${end}"
	echo -e "      ${opt}-l${arg} mode${reg}    : Controls exporting location settings.${end}"
	echo -e "      ${opt}-r${arg} mode${reg}    : Controls exporting practice settings.${end}"
	echo -e "      ${opt}-m${arg} mode${reg}    : Controls exporting managed groups settings (ie the groups the user can manage).${end}"
	echo -e "           ${reg}Possible modes for Location/Practice/Managed groups settings:${end}"
	echo -e "               ${lit}auto${reg}  : Export only entries used by users being exported (default).${end}"
	echo -e "               ${lit}none${reg}  : Do not export.${end}"
	echo -e "               ${lit}all${reg}   : Export all entries.${end}"
	echo
	echo -e "  ${reg}Plugins settings${end}"
	echo -e "      ${opt}-p${arg} mode${reg}    : Controls exporting user plugins settings.${end}"
	echo -e "           ${reg}Possible modes for plugins settings:${end}"
	echo -e "               ${lit}yes${reg}   : Export plugins settings of the users being exported (default).${end}"
	echo -e "               ${lit}no${reg}    : Do not export.${end}"
	echo
	
	[ -n "$1" ] && exit 1
	exit 0
}


#
# Check if a command line argument is one of the enumerated values
#
# $1: the option argument received in command line
# $2: space separated list of the valid values
# $3: the default value for unrecognized values
#
# output: the recognized valid or the default value
#
# return:  0  on valid or empty values
#          0< on error (unrecognized values)
#
function checkEnumOptions_() {
		local arg="$1"
		local values="$2"
		local default="$3"
		
		local v
		
		if [ -z "$arg" ] ; then
			echo "$default"
			return 0
		fi
		
		for v in $values; do
			if [ "$v" = "$arg" ] ; then
				echo -E "$v"
				return 0
			fi
		done
		
		echo "$default"
		return 1
}


#
# Creates a list from the provided values for an SQL IN clause
#
# $1: blank (space, new line) separated list of values
#
# output: list of values encoded for SQL IN clauses
#
function sqlList_() {
	local parValues="$1"
	
	local sqlList=""
	local isFirst=1
	local v
	
	for v in ${parValues} ; do
		if [ ${isFirst} -eq 1 ] ; then
			sqlList="'${v}'"
			isFirst=0
		else
			sqlList="$sqlList ,'${v}'"
		fi 
	done
	
	echo -E -n "${sqlList}"
}


#
# Returns user groups for individual users
#
# $1: space separated list of users
#
# output: blank (new line) separated list of (unique) user groups
#
# return:  0  on success
#          0< on error 
#
function getUserGroups_() {
	local userClause=$( sqlList_ "$1" )
	local query="select distinct AGroup from ${ACCESSTBL} where ${FldUserId} IN (${userClause})"
	
	# verbose_ "${query}" >&2
	${SqlSh} "${query}" -N	
}


#
# Generates expressions to be used in SQL where clauses to select requested users from the Users table
# 
# $1: Groups. All members in these groups will be selected
# $2: Individual users to select
#
# output: SQL expression to be used in where clauses (without the 'where' keyword) 
#
function getUsersWhereClause_() {
	local parGroups="$1"
	local parUsers="$2"

	local whereClause=""
	local inList=""

	if [ -n "${parGroups}" ] ; then
		inList=$( sqlList_ "${parGroups}" )
		if [ -n "${inList}" ] ; then 
			whereClause="${whereClause} OR AGroup IN (${inList})"
		fi
	fi
	if [ -n "${parUsers}" ] ; then
		inList=$( sqlList_ "${parUsers}" )
		if [ -n "${inList}" ] ; then 
			whereClause="${whereClause} OR ${FldUserId} IN (${inList})"
		fi
	fi

	echo -E -n "${whereClause# OR }"
}


#
# Generates expressions to be used in SQL where clauses to select requested groups from the Groups table
#
# $1: Groups to be exported with members
# $2: Groups to be exported individually (without members; data only)
#
# output: SQL expression to be used in where clauses (without the 'where' keyword) 
#
function getGroupsWhereClause_() {
	local parWholeGroups="$1"
	local parUserGroups="$2"
	
	local whereClause=""
	local inList=""

	inList=$( sqlList_ "${parWholeGroups} ${parUserGroups}" )
	if [ -n "${inList}" ] ; then
		whereClause="${whereClause} OR ID IN (${inList})"
	fi
	
	echo -E -n "${whereClause# OR }"
}


#
# Gets the list of selected users
#
# $1: 'true' indicates to list all users
# $2: SQL expression (for the where clause) to select particular users
#
# output: blank separated list of users
#
# return:  0  on success
#          0< on error 
#
function getUserList_() {
	local parAllUsers="$1"
	local parWhereClause="$2"

	local query="select distinct ${FldUserId} from ${ACCESSTBL}"

	if [ "${parAllUsers}" != "true" ] ; then
		if [ -z "${parWhereClause}" ] ; then
			verbose_ "No users provided" >&2
			return 0
		fi
		
		query="${query} WHERE ${parWhereClause}"
	fi

	verbose_ "${query}" >&2
	${SqlSh} "${query}" -N
}


#
# Gets the list of selected groups
#
# $1: 'true' indicates to list all groups
# $2: SQL expression (for the where clause) to select particular groups
#
# output: blank separated list of groups
#
# return:  0  on success
#          0< on error 
#
function getGroupList_() {
	local parAllGroups="$1"
	local parWhereClause="$2"

	local query="select distinct ID from ${GROUPSTBL}"

	if [ "${parAllGroups}" != "true" ] ; then
		if [ -z "${parWhereClause}" ] ; then
			verbose_ "No groups provided" >&2
			return 0
		fi

		query="${query} WHERE ${parWhereClause}"
	fi

	verbose_ "${query}" >&2
	${SqlSh} "${query}" -N
}


#
# Checks if a particular table exists in the database
#
# $1 : table name
#
# return:  0  if table exists
#          0< otherwise
#
function checkTable_() {
	local parTable="$1"
	
	local t
	
	t=$( ${SqlSh} "show tables like '${parTable}'" -N )
	
	[ "$t" = "${parTable}" ] && return 0
	return 1
}


#
# Export entries from a particular table
#
# $1: table name
# $2: 'true' indicates to export all entries
# $3: SQL expression (for the where clause) to select particular entries only
#
# return:  0  on success
#          0< on error 
#
function exportTable_() {
	local parTable="$1"
	local parAll="$2"
	local parWhereClause="$3"

	local err

	${MysqlDump} --no-data --skip-add-drop-table --skip-comments ${DBNAME} ${parTable} > "${WorkDumpDir}/create.${parTable}.sql"
	err=$?
	[ $err -ne 0 ] && return $err

	if [ "${parAll}" = "true" ] ; then
		${MysqlDump} --no-create-info --skip-comments --disable-extended-insert --skip-add-locks ${DBNAME} ${parTable} >> "${WorkDumpDir}/data.${parTable}.sql"
		return $?
	elif [ -n "${parWhereClause}" ] ; then
		${MysqlDump} --no-create-info --skip-comments --disable-extended-insert --skip-add-locks ${DBNAME} ${parTable} -w "${parWhereClause}" >> "${WorkDumpDir}/data.${parTable}.sql"
		return $?
	else
		warning_ "No entries to export from table ${parTable}"
	fi

	return 0
}


#
# Export User table entries
#
# $2: 'true' indicates to export all users
# $3: SQL expression (for the where clause) to export selected users only
#
# return:  0  on success
#          0< on error 
#
function exportUsersTable_() {
	local parAllUsers="$1"
	local parWhereClause="$2"
	
	info_ "Export Users table"
	exportTable_ "${ACCESSTBL}" "${parAllUsers}" "${parWhereClause}"
}


#
# Export Group table entries
#
# $2: 'true' indicates to export all groups
# $3: SQL expression (for the where clause) to export selected groups only
#
# return:  0  on success
#          0< on error 
#
function exportGroupsTable_() {
	local parAllGroups="$1"
	local parWhereClause="$2"

	info_ "Export Groups table"
	exportTable_ "${GROUPSTBL}" "${parAllGroups}" "${parWhereClause}"
}


#
# Export Relational table entries and entries from the Referenced table.
#
# Relational table:
#    Defines relation between users and entries in the Related table.
#    (eg.: UserLocation, UserPractice, PluginPerUser)
#
# Referenced table:
#    Entities referenced in the relational table (and having M to N relation with users)
#    (eg.: Location, Practice, )
#    Can be empty (like in case of plugins or managed groups), in this case only
#    the Relational table entries will be exported
#
# $1: Export mode
#     - none: export nothing
#     - all: export all entry from Referenced table (if Referenced table is not "")
#     - auto: export only entries referenced by the selected users (default)
# $2: Sql expression (in clause) to select users
# $3: the Referenced table
# $4: SQL name of the ID field identifying Referenced entities 
# $5: the Relational table
# $6: Label to refer the entities in output messages
#
# return:  0  on success
#          0< on error 
#
function exportReferenceTables_() {
	local parMode="$1"
	local parUsersClause="$2"
	local parTable="$3"
	local parIdField="$4"
	local parRelTable="$5"
	local parLabel="$6"
				
	local idList
	local idSqlList
	local idWhereClause

	if [ "${parMode}" = "none" ] ; then
		info_ "Skip exporting ${parLabel}s"
		return 0
	fi

	if [ -n "${parTable}" ] ; then
		checkTable_ "${parTable}"
		[ $? -ne 0 ] && error_ "${parTable} table does not exist" && return 1
	fi
	checkTable_ "${parRelTable}"
	[ $? -ne 0 ] && error_ "${parRelTable} table does not exist" && return 1
			
	# Relational table - select configuration for selected users only
	info_ "Export user ${parLabel} settings"
	if [ -n "${parUsersClause}" ] ; then
		exportTable_ "${parRelTable}" "${parAllGroups}" "${parUsersClause}"
		[ $? -ne 0 ] && error_ "Could not export user ${parLabel} settings" && return 1
	else
		info_ "There is no user ${parLabel} settings to export"
	fi

	# The referenced table
	[ -z "${parTable}" ] && return 0

	if [ "${parMode}" = "all" ] ; then
		info_ "Export all ${parLabel}s"
		exportTable_ "${parTable}" "true" ""
		[ $? -ne 0 ] && error_ "Could not export ${parLabel}s" && return 1
		return 0
	fi
	
	# export only 'referenced' records
	verbose_ "Collect ${parLabel}s to export"
	if [ -n "${parUsersClause}" ] ; then
		idList=$( ${SqlSh} "select distinct ${parIdField} from ${ACCESSTBL} natural left join ${parRelTable} WHERE ${parUsersClause}" -N )
	else
		info_ "There is no ${parLabel} to export"
		return 0
	fi

	idSqlList=$( sqlList_ "$idList" )
	if [ -z "${idSqlList}" ] ; then
		info_ "There is no ${parLabel} to export"
		return 0
	fi

	idWhereClause="${parIdField} IN (${idSqlList})"

	info_ "Export referenced ${parLabel}s"
	exportTable_ "${parTable}" "false" "${idWhereClause}"
	[ $? -ne 0 ] && error_ "Could not export ${parLabel}s" && return 1
	return 0
}


#
# Export location settings including referenced locations.
# (Location and UserLocation tables)
#
# $1: Export mode
#     - none: do not export location settings
#     - all : export all locations
#     - auto: export only locations referenced by the selected users (default)
# $2: Sql expression (in clause) to select users
#
# return:  0  on success
#          0< on error 
#
function exportLocations_() {
	local parMode="$1"
	local parUsersClause="$2"

	if [ "${parMode}" = "none" ] ; then
		info_ "Skip exporting locations"
		return 0
	fi
			
	exportReferenceTables_ "${parMode}" "${parUsersClause}" "${LOCATIONTBL}" "LOCATIONID" "${USERLOCATIONTBL}" "location"
}			


#
# Export practice settings including referenced practices
# (Practice and UserPractice tables)
#
# $1: Export mode
#     - none: do not export practice settings
#     - all : export all practices
#     - auto: export only practices referenced by the selected users (default)
# $2: Sql expression (in clause) to select users
#
# return:  0  on success
#          0< on error 
#
function exportPractices_() {
	local parMode="$1"
	local parUsersClause="$2"

	if [ "${parMode}" = "none" ] ; then
		info_ "Skip exporting practices"
		return 0
	fi

	exportReferenceTables_ "${parMode}" "${parUsersClause}" "${PRACTICETBL}" "PRACTICEID" "${USERPRACTICETBL}" "practice"
}


#
# Export plugins settings
# (PluginPerUser table)
#
# $1: Export mode
#     - no : do not export plugins settings
#     - yes: export plugin settings of the selected users (default)
# $2: Sql expression (in clause) to select users
#
# return:  0  on success
#          0< on error 
#
function exportPlugins_() {
	local parMode="$1"
	local parUsersClause="$2"

	if [ "${parMode}" = "no" ] ; then
		info_ "Skip exporting user plugins"
		return 0
	fi

	exportReferenceTables_ "" "${parUsersClause}" "" "PLUGINNAME" "${PLUGINPERUSERTBL}" "plugin"
}


#
# Export managed groups settings.
# (UserGroup table)
# The referenced groups will not be automatically added to the export
#
# $1: Export mode
#     - no : do not export managed groups settings
#     - yes: export managed groups settings of the selected users (default)
# $2: Sql expression (in clause) to select users
#
# return:  0  on success
#          0< on error 
#
function exportManagedGroups_() {
	local parMode="$1"
	local parUsersClause="$2"

	if [ "${parMode}" = "no" ] ; then
		info_ "Skip exporting user managed groups"
		return 0
	fi

	exportReferenceTables_ "" "${parUsersClause}" "" "ID" "${USERGROUPTBL}" "managed group"
}


#
# Copy the user directory for a particular user to the
# workdir repository
#
# $1 - user
#
function copyUserDir_() {
	local parUser="$1"

	local SrcDir
	local TargetDir
	local ErrorCode
	local ErrorOut

	SrcDir=$( ${REPOSITORYHANDLER_HOME}/bin/locateResource "${REPOSITORY_USERS}" "${parUser}" 2>&1 )
	ErrorCode=$?
	if [ ${ErrorCode} -ne 0 -o -z "${SrcDir}" ] ; then
		warning_ "Data directory could not be found for '${parUser}': ${SrcDir}"
		return 1
	fi

	TargetDir=$( ${REPOSITORYHANDLER_HOME}/bin/createLocation "${WorkUserRepo}" "${parUser}" new 2>&1 )
	ErrorCode=$?
	if [ ${ErrorCode} -ne 0 -o -z "${TargetDir}" ] ; then
		warning_ "No target directory could be created for '${parUser}': ${TargetDir}"
		return 1
	fi

	ErrorOut=$( cp -Rap "${SrcDir}" "${TargetDir}" 2>&1 )
	if [ $? -ne 0 ] ; then
		warning_ "Error copying data for '$u': ${ErrorOut}"
		return 1
	fi

	return 0
}



############################################################
############################################################
##
## Main
##


# default output file
optFile="eu.tar.gz"

# All users and groups
optExportAll="false"

# Groups with members
optExportGroups="false"
optGroupList=""
logGroupList=""

# Groups of individual users
optUserGroups="yes"

# Adding @system and @support settings
optAddSystem="auto"
optAddSupport="auto"

# Location, Practice and Managed Groups
optLocation="auto"
optPractice="auto"
optManagedGroups="auto"

# Plugin settings
optPlugins="yes"

setColors_


while getopts 'ahvc:s:x:l:r:m:p:g:f:u:' OPT; do
	case $OPT in
		c)
			if [ "$OPTARG" = "never" ] ; then
				clearColors_
			elif [ "$OPTARG" = "always" ] ; then
				setColors_
			fi
			;;
		h)
			usage_ ;;
		v)
			Verbose="true" ;;
		a)
			optExportAll="true" ;;
		g)
			optExportGroups="true"
			optGroupList="${optGroupList} ${OPTARG}"
			logGroupList="${logGroupList}; ${OPTARG}" ;;
		s)
			optAddSystem=$( checkEnumOptions_ "${OPTARG}" "auto no yes" "auto" )
			[ $? -ne 0 ] && warning_ "Invalid value provided for -${OPT}, using default mode ('auto')" 
			;;
		x)
			optAddSupport=$( checkEnumOptions_ "${OPTARG}" "auto no yes" "auto" )
			[ $? -ne 0 ] && warning_ "Invalid value provided for -${OPT}, using default mode ('auto')" 
			;;
		l)
			optLocation=$( checkEnumOptions_ "${OPTARG}" "auto none all" "auto" )
			[ $? -ne 0 ] && warning_ "Invalid value provided for -${OPT}, using default mode ('auto')" 
			;;
		r)
			optPractice=$( checkEnumOptions_ "${OPTARG}" "auto none all" "auto" )
			[ $? -ne 0 ] && warning_ "Invalid value provided for -${OPT}, using default mode ('auto')" 
			;;
		m)
			optManagedGroups=$( checkEnumOptions_ "${OPTARG}" "auto none all" "auto" )
			[ $? -ne 0 ] && warning_ "Invalid value provided for -${OPT}, using default mode ('auto')" 
			;;
		p)
			optPlugins=$( checkEnumOptions_ "${OPTARG}" "yes no" "yes" )
			[ $? -ne 0 ] && warning_ "Invalid value provided for -${OPT}, using default mode ('yes')" 
			;;
		u)
			optUserGroups=$( checkEnumOptions_ "${OPTARG}" "yes no" "no" )
			[ $? -ne 0 ] && warning_ "Invalid value provided for -${OPT}, using default mode ('no')" 
			;;
		f)
			optFile="$OPTARG" ;;
		*)
			usage_ "Invalid option: $OPT" ;;
	esac
done

shift $(($OPTIND-1))


if [ "${optExportAll}" = "true" ] ; then
	if [ -n "$1" ] ; then
		usage_ "No individual users/groups can be specified when using -a"
	fi

	[ "${optExportGroups}" = "true" ] && usage_ "Conflicting options: -a and -g"
			
	[ "${optAddSystem}" = "auto" ] && optAddSystem="yes"  
	[ "${optAddSupport}" = "auto" ] && optAddSupport="yes"  
		
else

	[ "${optExportGroups}" != "true" -a -z "$1" ] && usage_ "Missing argument(s): either -a, -g or id list should be specified"
	
	[ "${optAddSystem}" = "auto" ] && optAddSystem="no"
	[ "${optAddSupport}" = "auto" ] && optAddSupport="no"
fi 

verbose_ "Export All        : ${optExportAll}"
if [ "${optExportGroups}" = "true" ] ; then
	verbose_ "Export Groups     :${optGroupList}"
fi
verbose_ " -  @system       : ${optAddSystem}"
verbose_ " -  @support      : ${optAddSupport}"
verbose_ " -  Location      : ${optLocation}"
verbose_ " -  Practice      : ${optPractice}"
verbose_ " -  Managed Groups: ${optManagedGroups}"
verbose_ " -  Plugins       : ${optPlugins}"
verbose_ " -  User Groups   : ${optUserGroups}"
verbose_ " -  File          : ${optFile}"

if [ -n "$1" ] ; then
	verbose_ " -  Has individual users/groups : true"
else
	verbose_ " -  Has individual users/groups : false"
fi

FldUserId=""
FldUserId=$( $SqlSh "select f.DBNAME FROM $FIELDSTBL as f inner join $TABLESTBL as tab on f.COLID=tab.COLID WHERE tab.TABLENAME ='Access' AND tab.COLID = 'USID';" -N )
[ -z "${FldUserId}" ] && error_ "Database name of the user ID field could not be detected" && exit 1
verbose_ "User ID field: '$FldUserId'"

verbose_ "IDs: $@"


argUsers=""
argGroups=""

logUserList=""

while [ -n "$1" ] ; do
	group="${1#\#}"
	verbose_ "ID: $1 (${group})"
	if [ "${group}" != "$1" ] ; then
		argGroups="${argGroups} ${group}"
	else
		argUsers="${argUsers} $1"
	fi

	logUserList="${logUserList}; $1"
		
	shift
done

verbose_ "Individual users: ${argUsers}"
verbose_ "Individual groups: ${argGroups}"


if [ "${optUserGroups}" = "yes" -a -n "${argUsers}" ] ; then
	verbose_ "Add user groups to individual group list"
	
	userGroups=$( getUserGroups_ "${argUsers}" )
	[ $? -ne 0 ] && error_ "Could not get user groups for individual users"
	
	verbose_ "Add user groups to individual group list:"
	verbose_ ${userGroups}

	tmpGroups=$( for gr in ${argGroups} ${userGroups}; do echo $gr; done | sort -u )
	argGroups="${tmpGroups}"
	verbose_ "New individual group list:"
	verbose_ ${argGroups}
fi

# filters on provided groups or users
whereClauseUsers=""
# filters on provided groups
whereClauseGroups=""

if [ "${optExportAll}" != "true" ] ; then
	whereClauseUsers=$( getUsersWhereClause_ "${optGroupList}" "${argUsers}" )
	whereClauseGroups=$( getGroupsWhereClause_ "${optGroupList}" "${argGroups}" )
	
	verbose_ "Users where clause: ${whereClauseUsers}"
	verbose_ "Groups where clause: ${whereClauseGroups}"
fi

exportUsers=$( getUserList_ "${optExportAll}" "${whereClauseUsers}" )
[ $? -ne 0 ] && error_ "Could not collect the list of users" 

exportGroups=$( getGroupList_ "${optExportAll}" "${whereClauseGroups}" )
[ $? -ne 0 ] && error_ "Could not collect the list of groups" 

[ "${optAddSystem}" = "yes" ] && exportUsers="${exportUsers} @system"
[ "${optAddSupport}" = "yes" ] && exportUsers="${exportUsers} @support"

verbose_ "Exporting users: ["${exportUsers}"]"
verbose_ "Exporting groups: ["${exportGroups}"]"


verbose_ "Working Directory: ${WorkDir}"

mkdir "${WorkDir}"
mkdir "${WorkDumpDir}"
mkdir "${WorkUserRepo}"

touch "${WorkUserRepo}/repository.cfg"
touch "${WorkUserRepo}/repositorypart.cfg"


exportUsersTable_ "${optExportAll}" "${whereClauseUsers}"
exportGroupsTable_ "${optExportAll}" "${whereClauseGroups}"

# SQL '<fld> IN (<list>)' clause for all users to be exported
exportInClauseUsers=""
if [ -n "${exportUsers}" ] ; then
	sqlListUsers=$( sqlList_ "${exportUsers}" )
	if [ -n "${sqlListUsers}" ] ; then
		exportInClauseUsers="${FldUserId} IN (${sqlListUsers})"
	fi 
fi

exportLocations_ "${optLocation}" "${exportInClauseUsers}"
exportPractices_ "${optPractice}" "${exportInClauseUsers}"
exportPlugins_ "${optPlugins}" "${exportInClauseUsers}"
exportManagedGroups_ "${optManagedGroups}" "${exportInClauseUsers}"

info_ "Copying user/group data"

for u in $exportUsers ; do
	copyUserDir_ "${u}"
done

for g in $exportGroups; do
	copyUserDir_ "#${g}"
done


[ -f "$CONF/serverid.cfg" ] && . "$CONF/serverid.cfg"

cat <<-EOFINFO > "${InfoFile}"
	ServerId="${SELF_SERVER_ID}"
	ServerHostName="$(hostname)"
	DateTime="$(date +'%Y%m%d %H%M%S')"
	SystemIncluded="${optAddSystem}"
	SupportIncluded="${optAddSupport}"
	Locations="${optLocation}"
	Practices="${optPractice}"
	ManagedGroups="${optManagedGroups}"
	Plugins="${optPlugins}"
	UserGroups="${optUserGroups}"
EOFINFO

[ -n "${logGroupList}" ] && echo "GroupList=\"${logGroupList#; }\"" >> "${InfoFile}"
[ -n "${logUserList}" ] && echo "UserList=\"${logUserList#; }\"" >> "${InfoFile}"


info_ "Creating ${optFile}"

if [ "${optFile%.tar}" = "${optFile}" ] ; then
	verbose_ "Creating .tar.gz file"
	tar czf "${optFile}" -C "${WorkDir}/" .
	[ $? -ne 0 ] && error_ "Creating ${optFile} failed" && exit 1
else
	verbose_ "Creating .tar file"
		tar cf "${optFile}" -C "${WorkDir}/" .
	[ $? -ne 0 ] && error_ "Creating ${optFile} failed" && exit 1
fi

cat "${InfoFile}"

rm -rf ${WorkDir}

exit 0