#
# Component: tools
#
# Description:
#    Copies generic order attachments while creating special directory structure for our RIS 
#
# Usage: see below
#    
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

function usage_() {
	cat <<-EOFUSAGE
	    migrateAttachments [-f fromIndex] [-t toIndex] [-i]

		-i           : information mode - writes the min and max index (of Scheduler Order IDs) that may be used
		-f fromIndex : migrate only from this order ID, inclusive (if not present, no lower limit)
		-t toIndex   : migrate only until this order ID, inclusive (if not present, no upper limit)
		
	EOFUSAGE
	exit 1
}


INFOMODE="false"
FROM=""
TO=""

# reading the parameters
while getopts "f:t:i" OPT ; do
	case $OPT in
		i) INFOMODE="true" ;;
		f) FROM=$OPTARG ;;
		t) TO=$OPTARG ;;
		*) usage_  ;;
	esac
done

# if -i was used, just write the range of ORDER IDs
if [ $INFOMODE == "true" ] ; then
	MIN=`echo "select min( ORDERID ) from SchedulerOrder" | $MYSQL_HOME/bin/mysql -N imagemedical`
	MAX=`echo "select max( ORDERID ) from SchedulerOrder" | $MYSQL_HOME/bin/mysql -N imagemedical`
	echo "Order ID range from $MIN to $MAX" 
	exit 0
fi

#creating root directory - the target of migration
ROOTDIR="/home/medsrv/risDocs"
mkdir -p $ROOTDIR

echo "Collecting generic order attachments..."

# get the Orders from the Scheduler Order database (blocks are not real orders so we don't need them)
QUERY="select ORDERID from SchedulerOrder where BLOCK != 'yes'"

# apply the limits from the -t and/or -f parameters, if exists
if [ ! -z $FROM ] || [ ! -z $TO ] ; then
	if [ ! -z $FROM ] ; then
		QUERY=$QUERY" AND ORDERID >= $FROM "
	fi
	if [ ! -z $TO ] ; then
		QUERY=$QUERY" AND ORDERID <= $TO "
	fi
fi 

echo "Collect MySQL Query: $QUERY"
IDS=` echo "$QUERY" | $MYSQL_HOME/bin/mysql -N imagemedical `
for i in $IDS ; do
	echo
	echo "Order ID: $i"
	
	# get the hash directory of this Order ID, where the attachments may reside 
	HASH=` ~/component/utils/bin/getHashDirectory -i $i `
	echo "Hash: $HASH"
	
	# check if the directory exists
	if [ ! -d $ATTACHMENT_DATA/generic/order/repository/$HASH ] ; then
		echo "Directory not exists, no attachment here"
		continue
	fi
	
	# get the attachment files ( *._doctype_ files will not be used )
	cd $ATTACHMENT_DATA/generic/order/repository/$HASH
	# this handles filenames with spaces
	IFS=$'\n'
	ATTS=` ls --hide=*._doctype_ `
	# check if there's any attachment in the directory
	if [ ! -z "${ATTS}" ] ; then
		
		# get Acc.No.
		ACCNO=` echo "select ACCNO from SchedulerOrder where ORDERID=$i" | $MYSQL_HOME/bin/mysql -N imagemedical `
		if [ -z "${ACCNO}" ] || [ ${ACCNO} == "NULL" ] ; then
			ACCNO="empty"	
		fi
		echo "Acc.No: $ACCNO"
		
		# get Patient ID (a.k.a. MRN)
		PATID=` echo "select USERPATIENTID from SchedulerOrder natural left join SchedulerPatient where ORDERID=$i" | $MYSQL_HOME/bin/mysql -N imagemedical `
		if [ -z "${PATID}" ] || [ ${PATID} == "NULL" ] ; then
			PATID="empty"
		fi
		echo "Patient ID: $PATID"

		# for every attachment files:				
		for a in $ATTS ; do
			echo "Attachment: ${a}"	
		
			# create Unique ID from Order ID + name of attachment
			ATTID=$i-"${a}"
			echo "Unique ID: $ATTID"
		
			# get last modification time of the attachment
			LASTM=` stat -c '%y' "${a}" `
			# get only the first 19 characters (YYYY-MM-DD HH:MM:SS)
			# also get rid of the space in it (YYYY-MM-DD-HH:MM:SS) - I don't like spaces :)
			LASTMCUT=` echo "${LASTM:0:19}" | tr ' ' '-' `
			echo "Last modification time: $LASTMCUT"
			
			# create the directory (parents also if not exist)
			NEWDIR=${ROOTDIR}/${ATTID}/${PATID}/${ACCNO}/${LASTMCUT}
			mkdir -p $NEWDIR
			echo "Directory created: $NEWDIR"
			
			# copy the attachment
			cp "${a}" $NEWDIR
			echo "Attachment copied: ${a}"
		done
	fi
done